(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

exception Invalid_feature_set

let feature_sets = ["XG-2"; "XG"; "XG-NOPOS"]

type t = (string * string) list

let of_string s = [(s,s)]
let of_assoc_list al = al

let pos_tag dseq p =
  match FeatureGen.Source.get_info dseq.(p) with
      None -> raise FeatureGen.Feature_failure
    | Some atts -> 
	let tag = List.assoc "pos" atts in
	  FeatureGen.set_fs ([(tag,1.0)], false)

let chunk_fns lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     pos_tag;
     TextFeatures.n_gram_fn pos_tag [-2;-1];
     TextFeatures.n_gram_fn pos_tag [-1;0];
     TextFeatures.n_gram_fn pos_tag [0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2];
     TextFeatures.window_fns [pos_tag] [-2;-1;1;2]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let chunk_fns_2 lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     pos_tag;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [1;2];
     TextFeatures.n_gram_fn pos_tag [-2;-1;0];
     TextFeatures.n_gram_fn pos_tag [-1;0;1];    
     TextFeatures.n_gram_fn pos_tag [0;1;2];
     TextFeatures.n_gram_fn pos_tag [-2;-1];
     TextFeatures.n_gram_fn pos_tag [-1;0];
     TextFeatures.n_gram_fn pos_tag [0;1];
     TextFeatures.n_gram_fn pos_tag [1;2];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2];
     TextFeatures.window_fns [pos_tag] [-2;-1;1;2]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let chunk_fns_nopos lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "^[A-Z].*");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.node_f;
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns


let chunk_fns_nopos_2 lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "^[A-Z].*");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.node_f;
     TextFeatures.affixes_f 10;
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let chunk_fns_3 lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     pos_tag;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [1;2];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn pos_tag [-2;-1;0];
     TextFeatures.n_gram_fn pos_tag [-1;0;1];    
     TextFeatures.n_gram_fn pos_tag [0;1;2];
     TextFeatures.n_gram_fn pos_tag [-2;-1];
     TextFeatures.n_gram_fn pos_tag [-1;0];
     TextFeatures.n_gram_fn pos_tag [0;1];
     TextFeatures.n_gram_fn pos_tag [1;2];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [pos_tag] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let get_feature_fns feature_set data =
  match feature_set with
  | "XG" -> chunk_fns data
  | "XG-2" -> chunk_fns_2 data
  | "XG-3" -> chunk_fns_3 data
  | "XG-NOPOS" -> chunk_fns_nopos data
  | "XG-NOPOS-2" -> chunk_fns_nopos data
  | _ -> raise Invalid_feature_set
