(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

(** This is is the chunker runtime main.  It uses the Sgmlfront module/functor and thus assumes the
    input is SGML (or XML) with <lex> tags demarking tokens, possibly with a "pos" attribute indicating
    the part-of-speech.  The chunker can be run in two modes, one using provided parts of speech and
    the other not using part-of-speech.  In the latter case, part-of-speech tagging is not done at all
    and the chunker just uses words *)

exception Quit

module MySeq = Sequences.Make(ChunkerMgr)
module MySgmlfront = Sgmlfront.Make(MySeq)

let get_model model_file =
  let ic = open_in_bin model_file in
  let (crf_model: MySgmlfront.tag Crf.user_crf_model) = Marshal.from_channel ic in
  close_in ic;
  crf_model

let main () =
  let tst_dir = ref "" 
  and tst_regstr = ref ".*" 
  and model_file = ref "" 
  and outsuf = ref "" 
  and feature_set = ref ""
  and tri_gram = ref false
  and begin_encode = ref true
  and lex_dir = ref ""
  and input_file = ref ""
  and seq_boundary = ref "s"
  and server = ref false
  and output_file = ref "" in
  Arg.parse
    [("-input-dir", Arg.Set_string(tst_dir), "Input directory containing training files");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-filter", Arg.Set_string(tst_regstr), "Input regexp filter");
     ("-server", Arg.Set(server), "Run as a server");
     ("-input-file", Arg.Set_string(input_file), "A single input file (instead of -input-dir and -filter");
     ("-output-file", Arg.Set_string(output_file), "A single output file (instead of -outsuffix)");
     ("-outsuffix", Arg.Set_string (outsuf), "Suffix for tagged files");
     ("-tri-gram", Arg.Set(tri_gram), " Use tri-gram labels (best be same as training!)");
     ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set associated with task (POS, XG, XG-NOPOS, etc.)");
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    "Usage: chunk_decode.opt -input-dir <input dir> -model <model to use> -filter <regexp> -outsuffix <file suffix>";
  let crf_model = get_model !model_file in
  let tst_path,in_file = if !input_file = "" then !tst_dir,false else !input_file,true in
  let lex = 
    if !lex_dir = "" then None else 
      let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
	Some (TextFeatures.Lexicon.build_lexicons_txt paths) in
    if !server then
      try
	Printf.printf "READY\n"; flush stdout;
	while true do
	  let istr = read_line () in
	    match Str.split (Str.regexp "||") istr with
		[ifile;ofile] ->
		  MySgmlfront.run_on_sgml ~tri_gram:(!tri_gram) ~tst_filter:(Str.regexp (!tst_regstr)) ~add_begin:(!begin_encode) 
		  ~input_file:true ~output_file:ofile !seq_boundary ifile !outsuf crf_model !feature_set None;
		  Printf.printf "DONE\n"; flush stdout
	      | ["QUIT"] -> raise Quit
	      | _ -> Printf.printf "ERROR\n"
	done
      with Quit -> 
	exit (0)
    else
	MySgmlfront.run_on_sgml ~tri_gram:(!tri_gram) ~tst_filter:(Str.regexp (!tst_regstr)) ~add_begin:(!begin_encode) 
	  ~input_file:in_file ~output_file:!output_file !seq_boundary tst_path !outsuf crf_model !feature_set lex;;

main ();;
