
open GenseqUtils

let get_model model_file =
  let ic = open_in_bin model_file in
  let (crf_model: string Crf.user_crf_model) = Marshal.from_channel ic in
    close_in ic;
    crf_model

let decode_from_vecfile vecfile model outfile =
  let protoseqs = protoseqs_from_vectors vecfile in
  let tst_seqs,fa,la,fset = inst_seqs_from_protoseqs ~runtime:true 
			      ~tr_syms:(Some(model.Crf.crf_la,model.Crf.crf_fset))
			      protoseqs in
  let inv_la = FeatureGen.Alphabet.get_inverse la in
  let ostr = open_out outfile in
  let tscore = ref 0.
  and tcount = ref 0.
  and seqs_correct = ref 0 in
    Array.iter
      (fun seq -> 
	 let lscore = Viterbi.best_label_seq seq model.Crf.crf in
	 let slen = float_of_int (Array.length seq) in
	   tscore := !tscore +. (lscore *. slen);
	   tcount := !tcount +. slen;
	   if lscore = 1.0 then incr seqs_correct;
	   Array.iter 
	     (fun el -> 
		let lab = FeatureGen.InvAlphabet.get inv_la (FeatureGen.get_inst_label el) in
		let origlab = try FeatureGen.InvAlphabet.get inv_la (el.FeatureGen.crf_orig_label) with Not_found -> ":UNK:" in
		  Printf.fprintf ostr "%40s\t%s%40s\n" lab (if lab = origlab then "Cor" else "Incor") origlab) seq;
	   Printf.fprintf ostr "++\n") tst_seqs;
    close_out ostr;
    Printf.printf "---------------------------\n";
    Printf.printf "Instance/token-level accuracy: %f\n" (!tscore /. !tcount);
    Printf.printf "Sequence-level accuracy: %d/%d\n" !seqs_correct (Array.length tst_seqs);
    Printf.printf "---------------------------\n"

let main () =
  let tst_file = ref ""
  and model_file = ref ""
  and out_file = ref "" in
    Arg.parse
      [("-tst-file", Arg.Set_string(tst_file), " Input file with feature vectors ");
       ("-model-file", Arg.Set_string(model_file), " Model file to use" );
       ("-out-file", Arg.Set_string(out_file), " Output file for classifications")]
      (fun _ -> ())
      "Usage: genseq_decode(.opt) -tst-file ....";
    decode_from_vecfile !tst_file (get_model !model_file) !out_file;;

main ()
