
open GenseqUtils

let get_data edgep vecfile = 
  let protoseqs = protoseqs_from_vectors ~add_edge:edgep vecfile in
  let inst_seqs,fa,la,fset = inst_seqs_from_protoseqs protoseqs in
  let num_features = FeatureGen.Alphabet.get_size fa in
  let num_labels = FeatureGen.Alphabet.get_size la in 
    Printf.printf "\nTraining with %d sequences\n" (List.length protoseqs);
    inst_seqs,fa,la,num_features,num_labels,fset


let train_from_vecfile memmp edgep maxits uncompact vecfile =
  let inst_seqs,fa,la,num_features,num_labels,fset = get_data edgep vecfile in
  let inv_la = FeatureGen.Alphabet.get_inverse la in
  let model = 
    if memmp then
      Memm.train_memm ~verbose:true ~compact:(not uncompact) ~max_its:maxits 
	num_features num_labels inst_seqs 
    else
      Crf.train_crf ~verbose:true ~compact:(not uncompact) ~max_its:maxits 
	num_features num_labels inst_seqs  in
    Crf.create_user_model model fset la None 

let xvalidate edgep xval maxits vecfile =
  let inst_seqs,fa,la,num_features,num_labels,fset = get_data edgep vecfile in
    XValidate.xvalidate ~num_its:maxits inst_seqs num_features num_labels (1. /. (float_of_int xval)) xval

let main () =
  let tr_file = ref ""
  and model_file = ref ""
  and uncompact = ref false
  and gp = ref 100.
  and max_its = ref 1000
  and xval = ref (-1)
  and edge_p = ref false
  and intermediate = ref (-1) 
  and memm = ref false in
    Arg.parse
      [("-tr-file", Arg.Set_string(tr_file), " Input file with feature vectors ");
       ("-model-file", Arg.Set_string(model_file), " Model file to produce" );
       ("-gaussian-prior", Arg.Float (fun x -> Crf.invSigSquare := (1. /. (x *. x))), 
	"Gaussian prior variance");
       ("-uncompact", Arg.Set(uncompact), " Use uncompact (faster, but requires more memory) training");
       ("-edge", Arg.Set(edge_p), " Use edge feature");
       ("-xval", Arg.Set_int (xval), " X-validation");
       ("-max-iters", Arg.Set_int (max_its), " Max number of iterations (default 1000)");
       ("-memm", Arg.Set(memm), " Use Maximum Entropy Markov Model (instead of CRF)")]
      (fun _ -> ())
      "Usage: genseq_train(.opt) -tr-file ....";
    if !xval > 0 then 
      xvalidate !edge_p !xval !max_its !tr_file
    else
      let m = train_from_vecfile !memm !edge_p !max_its !uncompact !tr_file in
	Crf.marshal_crf !model_file m;;

main ()
       
