
let r1 = Str.regexp " +"
let r2 = Str.regexp ":"
let r3 = Str.regexp "[^0-9\\.]"

type protoseq = {lab: string;
		 features: (string * float * bool) array}

let protoseqs_from_vectors ?(add_edge=true) vecfile = 
  let istr = open_in vecfile in
  let seqs = ref [] in
  let cur_seq = ref [] in
  let rec get_features flist acc =
    match flist with
	a :: b ->
	  begin
	  try 
	    if a.[0] = '#' then
	      List.rev acc
	    else 
	      (match (Str.split r2 a) with
		   [fname;fval;edge_p] -> 
		     let afval = Str.global_replace r3 "" fval in
		     let aedge_p = Str.global_replace r3 "" edge_p in
		       get_features b ((fname,(float_of_string afval),((int_of_string aedge_p) = 1)) :: acc)
		 | [fname;fval] ->
		     let afval = Str.global_replace r3 "" fval in
		     let affloat = float_of_string afval in
		     let nfs = if (not add_edge) then ((fname,affloat,false) :: acc) else 
		       ((fname,affloat,false) :: (fname,affloat,true) :: acc) in
		       get_features b nfs
		 | [fname] ->
		     let nfs = if (not add_edge) then ((fname,1.0,false) :: acc) else 
		       ((fname,1.0,false) :: (fname,1.0,true) :: acc) in
		       get_features b nfs
		 | _ -> failwith "Invalid feature vector")
	  with Invalid_argument _  -> List.rev acc end
      | [] -> List.rev acc in
  let rec loop () =
    let li = (input_line istr) in
      match Str.split r1 li with
	  ["++"] | [""] | [] -> 
	    if not (!cur_seq = []) then (
	      (* cur_seq := {lab="END_STATE"; features=[|(":E:",1.0,true);(":U:",1.0,false)|]} :: !cur_seq; *)
	      let aseq = {lab="BEGIN_STATE"; features=[|(":U:",1.0,false)|]} :: (List.rev !cur_seq) in 
		seqs := (Array.of_list aseq) :: !seqs;
		cur_seq := []);
	    loop ()
	| label :: features ->
	    let nfeas = 
	      if add_edge then
		(":E:",1.0,true) :: (":U:",1.0,false) :: (get_features features [])
	      else (":U:",1.0,false) :: (get_features features []) in
	      cur_seq := {lab=label; features= Array.of_list nfeas} :: !cur_seq;
	      loop () in
    try 
      loop ()
    with End_of_file ->
      if not (!cur_seq = []) then
	seqs := (Array.of_list (List.rev !cur_seq)) :: !seqs;
      List.rev !seqs

let inst_seqs_from_protoseqs ?(runtime=false) ?(tr_syms=None) protoseqs =
  let la,fa,fset =
    if runtime then (
      match tr_syms with
	  Some(la,fset) -> (la,(FeatureGen.Alphabet.create ()),fset)
	| None -> failwith "Runtime must receive training symbols")
    else
      FeatureGen.Alphabet.create (), FeatureGen.Alphabet.create (), Hashtbl.create 10 in
  let nls = FeatureGen.Alphabet.get_size la in
  let inst_seqs = 
    Array.of_list
      (List.map
	 (fun proseq ->
	    let yprev = ref (-1) in
	    let inst_seq = Array.init (Array.length proseq) 
			     (fun _ -> FeatureGen.create_instance (if runtime then 0 else (-1))) in
	      Array.iteri
		(fun i position ->
		   let yp = FeatureGen.Alphabet.update la runtime position.lab in
		     FeatureGen.set_inst_label inst_seq.(i) yp;
		     FeatureGen.set_inst_orig_label inst_seq.(i) yp;
		     Array.iter
		       (fun (fname,fval,edgep) ->
			  let a_fname = FeatureGen.set_f fname in
			    if runtime then
			      FeatureGen.add_runtime_feature inst_seq fset i a_fname fval
			    else
			      FeatureGen.add_feature inst_seq fset i (if edgep then !yprev else (-1))
				yp a_fname fa fval) position.features;
		     yprev := yp) proseq;
	      inst_seq) protoseqs) in
    inst_seqs,fa,la,fset
