(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

(** This is main routine for launching text segmentation/labeling tasks. *)


module MySeq = Sequences.Make(KoreanNe)
module MySgmlfront = Sgmlfront.Make(MySeq)

let get_model model_file =
  let ic = open_in_bin model_file in
  let (crf_model: MySgmlfront.tag Crf.user_crf_model) = Marshal.from_channel ic in
  close_in ic;
  crf_model

let main () =
  let tst_dir = ref "" 
  and tst_regstr = ref ".*" 
  and model_file = ref "" 
  and input_file = ref ""
  and outsuf = ref "" 
(*   and lex_dir = ref "" *)
  and feature_set = ref ""
  and begin_encode = ref false
  and seq_boundary = ref "s"
  and tagset_file = ref ""
  and no_tok = ref false
  and white_space = ref false 
  and output_file = ref "" in
  Arg.parse
    [("-input-dir", Arg.Set_string(tst_dir), "Input directory containing training files");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-filter", Arg.Set_string(tst_regstr), "Input regexp filter");
     ("-tagset", Arg.Set_string(tagset_file), " Tagset file (for debugging, analysis, etc.)");
     ("-no-tok", Arg.Set(no_tok), " Use existing tokenization");
     ("-outsuffix", Arg.Set_string (outsuf), "Suffix for tagged files");
     ("-input-file", Arg.Set_string(input_file), "A single input file (instead of -input-dir and -filter");
     ("-output-file", Arg.Set_string(output_file), "A single output file (instead of -outsuffix)");
     ("-begin", Arg.Set(begin_encode), "Encode beginning of each phrase with begin label");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set to use");
     ("-white-space", Arg.Set(white_space), " Preserve regions of white space as tokens");
 (*     ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons"); *)
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    "Usage: segment -input-dir <input dir> -model <model to use> -filter <regexp> -outsuffix <file suffix> -lexicon-dir <lex dir>";
    let crf_model = get_model !model_file in
    let tagset = if !tagset_file = "" then None else Some (MySgmlfront.gather_tagset !tagset_file) in
    let tst_path,in_file = if !input_file = "" then !tst_dir,false else !input_file,true in
    let (tok_fn:(Sgml.xml -> Sgml.xml)) =  (* set up the tokenizer *)
      if (not !no_tok) then (
	print_endline "Tokenizing ..\n";
	Utf8tokenizer.tokenize ~white:(!white_space))
      else
	(fun s -> s) in
      MySgmlfront.run_on_sgml ~print_lex:false ~tagset:tagset ~toker:tok_fn ~tst_filter:(Str.regexp (!tst_regstr)) ~add_begin:(!begin_encode) ~input_file:in_file ~output_file:!output_file !seq_boundary tst_path !outsuf crf_model !feature_set None;;

main ();;
