(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

module MySeq = Sequences.Make(KoreanNe)
module MySgmlfront = Sgmlfront.Make(MySeq)
  
let main () =
  let tr_dir = ref ""
  and tr_regstr = ref ".*"
  and model_file = ref ""
  and tagset = ref "" 
(*   and lex_dir = ref "" *)
  and begin_encode = ref false
  and feature_set = ref ""
  and compact = ref false
  and max_its = ref 1000 
  and vp_train = ref false
  and induce_train = ref false
  and gp = ref 100.0
  and seq_boundary = ref "s"
  and no_tok = ref false
  and ind_beam_size = ref 100
  and ind_rounds = ref 30
  and white_space = ref false
  and intermediate = ref 0 in
  Arg.parse
    [("-input-dir", Arg.Set_string(tr_dir), "Input directory containing training files");
     ("-filter", Arg.Set_string(tr_regstr), "Input regexp filter");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-tagset", Arg.Set_string(tagset), "Tagset to train against");
     ("-gaussian-prior", Arg.Float (fun x -> Crf.invSigSquare := (1. /. (x *. x))), "Gaussian prior variance");
     ("-max-iters", Arg.Set_int (max_its), "Max number of iterations (default 1000)");
     ("-no-tok", Arg.Set(no_tok), " Use existing tokenization provided");
     ("-begin", Arg.Set(begin_encode), "Encode beginning of each phrase with begin label");
     ("-vp-train", Arg.Set(vp_train), " Use voted percepton training");
     ("-feature-induction", Arg.Set(induce_train), " Use feature induction");
     ("-induction-beam-size", Arg.Set_int(ind_beam_size), " Feature induction beam size");
     ("-induction-rounds", Arg.Set_int(ind_rounds), " Number of Feature induction rounds");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set to use");
     (* ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons"); *)
     ("-intermediate", Arg.Set_int(intermediate), " Dump out intermediate modesl (every n iterations)");
     ("-white-space", Arg.Set(white_space), " Preserve regions of white space as tokens");
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    "Usage: kne_train(.opt) -input-dir <input dir> -filter <regexp> -model <model file> ...";
    FeatureInduction.set_beam_size !ind_beam_size;
    FeatureInduction.set_num_rounds !ind_rounds;
  let ts = MySgmlfront.gather_tagset !tagset in
    (*
  let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
  let lex = TextFeatures.Lexicon.build_lexicons_txt paths in  *)
  let inter_f = if !intermediate > 0 then Some !model_file else None in
  let ttype = if !induce_train then MySgmlfront.CrfInduce else if !vp_train then MySgmlfront.Vp else MySgmlfront.Crf in
  let (tok_fn:(Sgml.xml -> Sgml.xml)) =  (* set up the tokenizer *)
    if (not !no_tok) then 
      Utf8tokenizer.tokenize ~white:(!white_space)
    else
      (fun s -> s) in
  let user_crf = 
    MySgmlfront.train_from_sgml ~intermediate:(if !intermediate > 0 then !intermediate else 50) ~toker:tok_fn ~i_file_root:inter_f ~train_type:ttype ~compact:(!compact) ~max_its:(!max_its) 
      ~tr_filter:(Str.regexp (!tr_regstr)) 
      ~add_begin:(!begin_encode) !seq_boundary !tr_dir !feature_set ts None in
    Crf.marshal_crf !model_file user_crf;;

main ();;
