exception Done

let regexp korean_affix = ( ( 51221 49345 ) )

let regexp korean_affix_app = ( (korean_affix xml_letter* korean_affix)
			      | (xml_letter+ korean_affix)
			      | (korean_affix xml_letter+) )

let regexp number = [ '0'-'9' ]+

let wrap chars = Sgml.Element("lex",[],[Sgml.PCData chars])

let rec token = lexer
  | korean_affix -> Ulexing.utf8_lexeme lexbuf
  | eof -> raise Done
  | xml_letter -> Ulexing.utf8_lexeme lexbuf


let rec full_token2 tlist = lexer
  | korean_affix_app -> 
      let tt = ref [] in
      let nbuf = Ulexing.from_utf8_string (Ulexing.utf8_lexeme lexbuf) in
	begin
	try
	  while true do
	    let s = token nbuf in
	      tt := s :: !tt
	  done
	with Done ->
	  let sbuf = Buffer.create 4 in
	    List.iter (fun t -> 
			 if (Utf8.compute_len t 0 (String.length t)) < 2 then 
			   Buffer.add_string sbuf t
			 else
			   let s = Buffer.contents sbuf in
			     Buffer.clear sbuf;
			     if String.length s > 0 then
			       tlist := (wrap s) :: !tlist;
			     tlist := (wrap t) :: !tlist) (List.rev !tt);
	    if (Buffer.length sbuf) > 0 then
	      tlist := (wrap (Buffer.contents sbuf)) :: !tlist;
	end
  | xml_letter+ -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
  | number -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
  | eof -> raise Done
  | " " -> ()
  | _ -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist


let tokenize_utf8_string s =
  let toks = ref [] in
  let lexbuf = Ulexing.from_utf8_string s in
    try
      while true do full_token2 toks lexbuf done; List.rev !toks
    with Done -> List.rev !toks

(* note - may run out of memory on very long files as we use the stack *)
let tokenize_sgml ?(white=false) sg_l =
  let rec tokenize_sgml_x sg_l acc =
    match sg_l with
	[] -> acc
      | Sgml.Element(l,a,c) :: rest ->
	  let toked_content = tokenize_sgml_x c [] in
	    tokenize_sgml_x rest (Sgml.Element(l,a,toked_content) :: acc)
      | Sgml.PCData c :: rest ->
	  tokenize_sgml_x rest (List.rev_append (tokenize_utf8_string c) acc) in
    tokenize_sgml_x sg_l []

let tokenize sg = List.hd (tokenize_sgml [sg])
