exception Done

let r1 = Str.regexp "[' ' \n \r]+"


let wrap chars = Sgml.Element("lex",[],[Sgml.PCData chars])
let regexp latin_wd = [ 'A'-'Z' 'a'-'z' ]+
let regexp number = [ '0'-'9' ]+
let regexp ws = ["\n" " "]+

let rec char_tok tlist = lexer
      | latin_wd -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
      | number -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
      | xml_letter ->
	  tlist := wrap (Ulexing.utf8_lexeme lexbuf) :: !tlist
      | eof -> raise Done
      | ws -> tlist := Sgml.PCData (Ulexing.utf8_lexeme lexbuf) :: !tlist
      | _ -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist

(* this version preserves whitespace tokens *)
let rec char_tok_ws tlist = lexer
      | latin_wd -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
      | number -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
      | xml_letter ->
	  tlist := wrap (Ulexing.utf8_lexeme lexbuf) :: !tlist
      | eof -> raise Done
      | ws -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist
      | _ -> tlist := (wrap (Ulexing.utf8_lexeme lexbuf)) :: !tlist

let tokenize_utf8_string ?(white=false) s =
  let toks = ref [] in
  let lexbuf = Ulexing.from_utf8_string s in
  let tfn = if white then char_tok_ws else char_tok in
    try
      while true do tfn toks lexbuf done; List.rev !toks
    with Done -> List.rev !toks

let tokenize_sgml ?(white=false) sg_l =
  let rec tokenize_sgml_x sg_l acc =
    match sg_l with
	[] -> List.rev acc
      | Sgml.Element(l,a,c) :: rest ->
	  let toked_content = tokenize_sgml_x c [] in
	    tokenize_sgml_x rest (Sgml.Element(l,a,toked_content) :: acc)
      | Sgml.PCData c :: rest ->
	  tokenize_sgml_x rest (List.rev_append (tokenize_utf8_string ~white:white c) acc) in
    tokenize_sgml_x sg_l []

let tokenize ?(white=false) sg = 
  let r = List.hd (tokenize_sgml ~white:white [sg]) in
    r
