open Camomile
let r1 = Str.regexp "[' ' \n \r]+"

module M = URe.Make(UTF8)

type affix = Affix of UTF8.t * UTF8.index | Null

let (prefixes: M.compiled_regexp ref) = ref (M.compile (`Rep(`OneChar)))
let (suffixes: M.compiled_regexp ref) = ref (M.compile (`Rep(`OneChar)))

let build_affix_regexp ?(prefix=true) prefix_file =
  let (wr_fn: URe.regexp -> URe.regexp) = 
    if prefix then 
      (fun x -> (`Seq(`BoS,x))) 
    else 
      (fun x -> (`Seq(x,`EoS))) in
  let istr = open_in prefix_file in
  let (local_res: URe.regexp list ref) = ref [] in
  let rec loop () =
    let l = Str.split r1 (input_line istr) in
    let chars = List.map (fun cp -> UChar.chr_of_uint (int_of_string cp)) l in
      local_res := (wr_fn (`String(chars))) :: !local_res;
      loop () in
    try loop () 
    with End_of_file ->
      let final_reg = List.fold_left 
			(fun (c:URe.regexp) (x:URe.regexp) -> 
			   `Alt(x,c)) 
			(List.hd !local_res)
			(List.tl !local_res) in
	M.compile final_reg

let initialize pfile sfile =
  prefixes := build_affix_regexp ~prefix:true pfile;
  suffixes := build_affix_regexp ~prefix:false sfile

let wrap chars = Sgml.Element("lex",[],[Sgml.PCData chars])


let sub_string utf8string ind1 ind2 = 
  assert ((UTF8.compare_index utf8string ind1 ind2) <= 0);
  let sbuf = UTF8.Buf.create 4 in
  let rec loop cind =
    if ((UTF8.compare_index utf8string cind ind2) <= 0) then (
      UTF8.Buf.add_char sbuf (UTF8.look 
				utf8string 
				cind);
      loop (UTF8.next utf8string cind)) in
  loop ind1;
  UTF8.Buf.contents sbuf
		 

let subtokenize_word word =
  let last = UTF8.last word in
  let first = UTF8.first word in
    if ((UTF8.compare_index word first last) < 0) then (
      let prefix = 
	match M.regexp_match !prefixes word (UTF8.first word) with
	    Some [| (Some s) |] -> 
	      let t,i1,i2 = M.SubText.context s in
		Affix((M.SubText.excerpt s),i2)
	  | _ -> Null in
      let suffix = 
	match M.regexp_match !suffixes word (match prefix with Affix(_,s) -> s | Null -> (UTF8.first word)) with
	    Some [| (Some s) |] ->
	      let t,i1,i2 = M.SubText.context s in
		Affix ((M.SubText.excerpt s),i1)
	  | _ -> Null in
	match prefix, suffix with
	    Null,Null -> [word]
	  | Affix(pre,i), Null when (UTF8.compare_index word i last) <= 0 -> [pre; (sub_string word i last)]
	  | Affix(pre,i), Null -> [word]
	  | Null,Affix(suf,i) -> 
	      if (UTF8.compare_index word first i) <= 0 then [suf]
	      else [(sub_string word first (UTF8.prev word i)); suf]
	  | Affix(pre,pi),Affix(suf,si) when (UTF8.compare_index word pi si) < 0 -> 
	      [pre; (sub_string word pi (UTF8.prev word si)); suf]
	  | Affix(pre,pi),Affix(suf,si) when (UTF8.compare_index word pi si) = 0 -> [pre;suf]
	  | Affix(pre,pi),Affix(_,_) -> [pre; (sub_string word pi last)] (* favor prefixes *) )
    else [word]
	  
let tokenize_utf8_string_with_affixes s =
  let word_toks = Str.split r1 s in
    List.rev (
    List.fold_left
      (fun acc word ->
(*	 (try Printf.printf "word: %s   length: %d\n" word (UTF8.length word) with
	     Invalid_argument _ -> Printf.printf " bad word: %s\n" word);
	   flush stdout;  *)
	 let subtoks = List.map wrap (subtokenize_word word) in  
	   List.rev_append subtoks acc)
      [] word_toks)


(* tail-recursive(mostly) version *)
let tokenize_sgml ?(white=false) sg_l =
  let rec tokenize_sgml_x sg_l acc =
    match sg_l with
	[] -> List.rev acc
      | Sgml.Element(l,a,c) :: rest ->
	  let toked_content = tokenize_sgml_x c [] in
	    tokenize_sgml_x rest (Sgml.Element(l,a,toked_content) :: acc)
      | Sgml.PCData c :: rest ->
	  tokenize_sgml_x rest (List.rev_append (tokenize_utf8_string_with_affixes c) acc) in
    tokenize_sgml_x sg_l []


(* (* this is the non tail-recursive version ... probably faster on smaller documents *)
let tokenize_sgml ?(white=false) sg_l =
  let rec tokenize_sgml_x sg_l =
    match sg_l with
	[] -> []
      | Sgml.Element(l,a,c) :: rest ->
	  let toked_content = tokenize_sgml_x c in
	    Sgml.Element(l,a,toked_content) :: (tokenize_sgml_x rest)
      | Sgml.PCData c :: rest ->
	  let t1 = (tokenize_utf8_string_with_affixes c) in
	    t1 @ (tokenize_sgml_x rest) in
    tokenize_sgml_x sg_l
*)

let tokenize sg = List.hd (tokenize_sgml [sg])
