(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

exception Invalid_feature_set

let feature_sets = ["NE-2"; "NE"; "NE-NOPOS"; "NE-NOPOS-2"]

type t = (string * string) list

let of_string s = [(s,s)]
let of_assoc_list s = s

let pos_tag dseq p =
  match FeatureGen.Source.get_info dseq.(p) with
      None -> raise FeatureGen.Feature_failure
    | Some atts -> 
	let tag = List.assoc "pos" atts in
	  FeatureGen.set_fs ([(tag,1.0)], false)

let ne_fns lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     pos_tag;
     TextFeatures.n_gram_fn pos_tag [-2;-1];
     TextFeatures.n_gram_fn pos_tag [-1;0];
     TextFeatures.n_gram_fn pos_tag [0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2];
     TextFeatures.window_fns [pos_tag] [-2;-1;1;2]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let ne_fns_2 lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*$");
     TextFeatures.regexp_f_gen "INITCAP_ALPHA" (Str.regexp "[A-Z][a-z]*$");
     TextFeatures.regexp_f_gen "ALL_CAPS" (Str.regexp "[A-Z]+$");
     TextFeatures.regexp_f_gen "CAPS_MIX" (Str.regexp "[A-Za-z]+$");
     TextFeatures.regexp_f_gen "HAS_DIGIT" (Str.regexp ".*[0-9].*$");
     TextFeatures.regexp_f_gen "SINGLE_DIGIT" (Str.regexp "[0-9]$");
     TextFeatures.regexp_f_gen "DOUBLE_DIGIT" (Str.regexp "[0-9][0-9]$");
     TextFeatures.regexp_f_gen "FOUR_DIGIT" (Str.regexp "[0-9][0-9][0-9][0-9]$");
     TextFeatures.regexp_f_gen "NAT_NUM" (Str.regexp "[0-9]+$");
     TextFeatures.regexp_f_gen "REAL_NUM" (Str.regexp "[0-9]\\.[0-9]+$");
     TextFeatures.regexp_f_gen "ALPHA_NUM" (Str.regexp "[0-9A-Za-z]+$");
     TextFeatures.regexp_f_gen "ROMAN" (Str.regexp "[lcximvLCXIMV]+$");
     TextFeatures.regexp_f_gen "HAS_DASH" (Str.regexp ".*-.*$");
     TextFeatures.regexp_f_gen "INIT_DASH" (Str.regexp "^-.*$");
     TextFeatures.regexp_f_gen "END_DASH" (Str.regexp ".*-$");
     TextFeatures.regexp_f_gen "PUNCT" (Str.regexp "^[^A-Za-z0-9]+$");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.affixes_f 4;
     pos_tag;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];    
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1;2];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [1;2];
     TextFeatures.n_gram_fn pos_tag [-2;-1;0];
     TextFeatures.n_gram_fn pos_tag [-1;0;1];    
     TextFeatures.n_gram_fn pos_tag [0;1;2];
     TextFeatures.n_gram_fn pos_tag [-2;-1];
     TextFeatures.n_gram_fn pos_tag [-1;0];
     TextFeatures.n_gram_fn pos_tag [0;1];
     TextFeatures.n_gram_fn pos_tag [1;2];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2];
     TextFeatures.window_fns [pos_tag] [-2;-1;1;2];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				(Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns


let ne_fns_3 lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*$");
     TextFeatures.regexp_f_gen "INITCAP_ALPHA" (Str.regexp "[A-Z][a-z]*$");
     TextFeatures.regexp_f_gen "ALL_CAPS" (Str.regexp "[A-Z]+$");
     TextFeatures.regexp_f_gen "CAPS_MIX" (Str.regexp "[A-Za-z]+$");
     TextFeatures.regexp_f_gen "HAS_DIGIT" (Str.regexp ".*[0-9].*$");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.affixes_f 4;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];    
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1;2];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [1;2];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				(Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let ne_fns_nopos lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z].*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.node_f;
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let get_feature_fns feature_set data =
  match feature_set with
  | "NE" -> ne_fns data
  | "NE-2" -> ne_fns_2 data
  | "NE-NOPOS-2" -> ne_fns_3 data
  | "NE-NOPOS" -> ne_fns_nopos data
  | _ -> raise Invalid_feature_set
