(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

module MySeq = Sequences.Make(NeMgr)
module MySgmlfront = Sgmlfront.Make(MySeq)
  
let main () =
  let tr_dir = ref ""
  and tr_regstr = ref ".*"
  and model_file = ref ""
  and tagset = ref "" 
  and begin_encode = ref false
  and feature_set = ref ""
  and uncompact = ref false
  and max_its = ref 1000 
  and vp_train = ref false
  and hybrid_train = ref false
  and gp = ref 100.0
  and tri_gram = ref false 
  and seq_boundary = ref "s"
  and brief = ref false
  and input_file = ref ""
  and lex_dir = ref ""
  and intermediate = ref (-1) in
    Arg.parse
      [("-input-dir", Arg.Set_string(tr_dir), "Input directory containing training files");
       ("-filter", Arg.Set_string(tr_regstr), "Input regexp filter");
       ("-input-file", Arg.Set_string(input_file), "A single input file (instead of using -input-dir and -filter)");
       ("-model", Arg.Set_string(model_file), "Output model file");
       ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons");
       ("-tagset", Arg.Set_string(tagset), "Tagset to train against");
       ("-uncompact", Arg.Set(uncompact), "Compact training (slower, but conserves memory)");
       ("-gaussian-prior", Arg.Float (fun x -> Crf.invSigSquare := (1. /. (x *. x))), "Gaussian prior variance");
       ("-max-iters", Arg.Set_int (max_its), "Max number of iterations (default 1000)");
       ("-vp-train", Arg.Set(vp_train), " Use voted percepton training");
       ("-brief", Arg.Set(brief), " Omit printing of detailed information during training");
       ("-hybrid-train", Arg.Set(hybrid_train), " Use hybrid vp/crf training");
       ("-tri-grams", Arg.Set(tri_gram), " Use label tri-grams (and bi-grams)");
       ("-feature-set", Arg.Set_string(feature_set), " Feature set to use");
       ("-intermediate", Arg.Set_int(intermediate), " Number of iterations per intermediate model");
       ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    ("Current allowed feature sets: "^(String.concat " " NeMgr.feature_sets));
  let ts = MySgmlfront.gather_tagset !tagset in
  let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
  let lex = TextFeatures.Lexicon.build_lexicons_txt paths in
  let inter_f = if !intermediate > 0 then Some !model_file else None in
    Printf.printf "\n Training from sgml";
    let ttype = 
      if !hybrid_train then 
	MySgmlfront.Hybrid 
      else if !vp_train then 
	MySgmlfront.Vp 
      else MySgmlfront.Crf in
    let tr_path,in_file = if !input_file = "" then !tr_dir,false else !input_file,true in
    let user_crf = 
      MySgmlfront.train_from_sgml 
	~verbose:(not !brief)
	~intermediate:(!intermediate) 
	~i_file_root:inter_f 
	~train_type:ttype 
	~compact:(not (!uncompact)) 
	~max_its:(!max_its) 
	~tri_gram:(!tri_gram)
	~tr_filter:(Str.regexp (!tr_regstr)) 
	~add_begin:(!begin_encode) 
	~input_file:in_file (* specify whether tr_path is a file or directory *)
	!seq_boundary tr_path !feature_set ts (Some lex) in
  Crf.marshal_crf !model_file user_crf;;

main ();;
