(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

let feature_sets = ["POS"; "POS-2"; "POS-3"; "POS-4"; "POS-5"]

type t = (string * string) list

let of_string s = [(s,s)]
let of_assoc_list al = al 


let entity_type dseq p =
  try
    let info = 
      match FeatureGen.Source.get_info dseq.(p) with 
	  Some i -> i 
	| None -> raise Not_found in
    let ent = List.assoc "entity" info in
      FeatureGen.set_fs ([("ENTITY",1.0);(("ET:"^ent),1.0)], false)
  with Not_found -> raise FeatureGen.Feature_failure


let plural_regexp = Str.regexp "^.*s$"
let simple_past_regexp = Str.regexp "^.*ed$"

let pos_fns lex =
  let main_fns = 
    [TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "E_S" (Str.regexp ".*[s|S]$");
     TextFeatures.regexp_f_gen "E_SS" (Str.regexp ".*[s|S][s|S]$");
     TextFeatures.regexp_f_gen "E_ED" (Str.regexp ".*[e|E][d|D]$");
     TextFeatures.regexp_f_gen "E_ING" (Str.regexp ".*[i|I][n|N][g|G]$");
     TextFeatures.regexp_f_gen "E_LY" (Str.regexp ".*[l|L][y|Y]$");
     TextFeatures.regexp_f_gen "E_ED" (Str.regexp ".*[l|L][y|Y]$");
     TextFeatures.regexp_f_gen "E_BLE" (Str.regexp ".*[b|B][l|L][e|E]$");
     TextFeatures.regexp_f_gen "E_IC" (Str.regexp ".*[i|I][c|C]$");
     TextFeatures.regexp_f_gen "E_AL" (Str.regexp ".*[a|A][l|L]$");
     TextFeatures.regexp_f_gen "E_US" (Str.regexp ".*[u|U][s|S]$");
     TextFeatures.regexp_f_gen "E_IVE" (Str.regexp ".*[i|I][v|V][e|E]$");
     TextFeatures.regexp_f_gen "E_TS" (Str.regexp ".*[t|T][s|S]$");
     TextFeatures.regexp_f_gen "E_IZE" (Str.regexp ".*[i|I][z|Z][e|E]$");
     TextFeatures.regexp_f_gen "E_ATE" (Str.regexp ".*[a|A][t|T][e|E]$");
     TextFeatures.regexp_f_gen "E_ARY" (Str.regexp ".*[a|A][r|R][y|Y]$");
     TextFeatures.regexp_f_gen "E_ANS" (Str.regexp ".*[a|A][n|N][s|S]$");
     TextFeatures.regexp_f_gen "E_MEN" (Str.regexp ".*[m|M][e|E][n|N]$");
     TextFeatures.regexp_f_gen "B_HYPH" (Str.regexp "^-.*");
     TextFeatures.regexp_f_gen "B_0" (Str.regexp "^0.*");
     TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2]] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns


let pos_fns_2 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-2;-1;1;2];
     TextFeatures.affixes_f 5] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

let pos_fns_3 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "[A-Z]*");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

let pos_fns_4 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "[A-Z]*");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "HASCAP" (Str.regexp ".*[A-Z].*");
     TextFeatures.regexp_f_gen "HASNUMBER" (Str.regexp ".*[0-9].*");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;2]; (* jump ahead one *)
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  [(TextFeatures.lexicon_membership_f lex);
	   (TextFeatures.window_fns [(TextFeatures.lexicon_membership_f lex)] [-3;-2;-1;1;2;3])]
	  @ main_fns

(* 9/26/05 - This version includes Stanford paper features as well as nam*)
let pos_fns_5 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [-1;0;1]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "CAPDIGHYPH"  (* check for F/A-18 *)
       (Str.regexp "^[^a-z]*[0-9]*[^a-z]*-[^a-z]*[0-9]+[^a-z]*$\\|^[^a-z]*[0-9]+[^a-z]*-[^a-z]*[0-9]*[^a-z]*$");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  [(TextFeatures.lexicon_membership_f lex);
	   (TextFeatures.window_fns [(TextFeatures.lexicon_membership_f lex)] [-3;-2;-1;1;2;3])]
	  @ main_fns

let pos_fns_6 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "CAPDIGHYPH"  (* check for F/A-18 *)
       (Str.regexp "^[^a-z]*[0-9]*[^a-z]*-[^a-z]*[0-9]+[^a-z]*$\\|^[^a-z]*[0-9]+[^a-z]*-[^a-z]*[0-9]*[^a-z]*$");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> (TextFeatures.lexicon_membership_f lex) :: main_fns

(* this is pos_fns_3 with NE, no other differences at all *)
let pos_fns_7 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "[A-Z]*");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

(* this with NE around current word, and no ORG-hack feature *)
let pos_fns_9 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [-1;0;1]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "[A-Z]*");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

(* no contextual features at all *)
let pos_fns_base lex =
  let main_fns = 
    [TextFeatures.wd_f;
     (* TextFeatures.edge_f; *)
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

let r1 = Str.regexp "^[A-Z].*$"
let r2 = Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$"

let org_hack dseq pos =
  match FeatureGen.Source.get_obs dseq.(pos) with
      Some t ->
	if Str.string_match r1 t 0 then
	  TextFeatures.regexp_f_gen "CORP_WD" r2 dseq pos
	else raise FeatureGen.Feature_failure
    | None -> raise FeatureGen.Feature_failure

(* no contextual features at all *)
let pos_fns_base2 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     (* TextFeatures.edge_f; *)
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "SINGLECAP" (Str.regexp "^.+[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "MULTICAP" (Str.regexp "^.+[A-Z].*[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "HASPERIOD" (Str.regexp ".*\\..*");
     TextFeatures.regexp_f_gen "HASNUM" (Str.regexp ".*[0-9].*");
     TextFeatures.regexp_f_gen "NOALPH" (Str.regexp "^[^A-z]+$");
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns


(* no contextual features at all *)
let pos_fns_base3 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     (* TextFeatures.edge_f; *)
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "SINGLECAP" (Str.regexp "^.+[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "MULTICAP" (Str.regexp "^.+[A-Z].*[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "HASPERIOD" (Str.regexp ".*\\..*");
     TextFeatures.regexp_f_gen "HASNUM" (Str.regexp ".*[0-9].*");
     TextFeatures.regexp_f_gen "NOALPH" (Str.regexp "^[^A-z]+$");
     org_hack;
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

(* this is pos_fns_7 some additional features from the SVM pape *)
let pos_fns_11 lex =
  let main_fns = 
    [TextFeatures.wd_f;
     TextFeatures.edge_f;
     TextFeatures.node_f;
     TextFeatures.window_fns [entity_type] [0]; (* named entity *)
     TextFeatures.regexp_f_gen "INITCAP" (Str.regexp "^[A-Z][^A-Z]*$");
     TextFeatures.regexp_f_gen "SINGLECAP" (Str.regexp "^.+[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "MULTICAP" (Str.regexp "^.+[A-Z].*[A-Z].*$"); (* more than one not at begin *)
     TextFeatures.regexp_f_gen "ALLCAP" (Str.regexp "^[A-Z]*$");
     TextFeatures.regexp_f_gen "HASHYPHEN" (Str.regexp ".*-.*");
     TextFeatures.regexp_f_gen "HASPERIOD" (Str.regexp ".*\\..*");
     TextFeatures.regexp_f_gen "HASNUM" (Str.regexp ".*[0-9].*");
     TextFeatures.regexp_f_gen "NOALPH" (Str.regexp "^[^A-z]+$");
     TextFeatures.n_gram_fn TextFeatures.wd_f [-2;-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
     TextFeatures.n_gram_fn TextFeatures.wd_f [0;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;1];
     TextFeatures.n_gram_fn TextFeatures.wd_f [-1;0;1];
     TextFeatures.window_fns [TextFeatures.wd_f] [-3;-2;-1;1;2;3];
     TextFeatures.window_fns [TextFeatures.regexp_f_gen "COMP_WD" 
				 (Str.regexp "Ltd[\\.]?$\\|Corp[\\.]?$\\|Co[\\.]$\\|Inc[\\.]$")] [1;2;3;4];
     TextFeatures.affixes_f 10] in
    match lex with 
	None -> main_fns
      | Some lex -> 
	  (TextFeatures.lexicon_membership_f lex) :: main_fns

let get_feature_fns feature_set data =
  match feature_set with
  | "POS" -> pos_fns data
  | "POS-2" -> pos_fns_2 data
  | "POS-3" -> pos_fns_3 data
  | "POS-4" -> pos_fns_4 data
  | "POS-5" -> pos_fns_5 data
  | "POS-6" -> pos_fns_6 data
  | "POS-7" -> pos_fns_7 data
  | "POS-9" -> pos_fns_9 data
  | "POS-11" -> pos_fns_11 data
  | "POS-base" -> pos_fns_base data
  | "POS-base2" -> pos_fns_base2 data
  | "POS-base3" -> pos_fns_base3 data
  | _ -> raise Not_found
