(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)

(** This is is the chunker runtime main.  It uses the Sgmlfront module/functor and thus assumes the
    input is SGML (or XML) with <lex> tags demarking tokens, possibly with a "pos" attribute indicating
    the part-of-speech.  The chunker can be run in two modes, one using provided parts of speech and
    the other not using part-of-speech.  In the latter case, part-of-speech tagging is not done at all
    and the chunker just uses words *)

module MySeq = Sequences.Make(PosMgr)
module MySgmlfront = Sgmlfront.Make(MySeq)

let get_model model_file =
  let ic = open_in model_file in
  let (crf_model: MySgmlfront.tag Crf.user_crf_model) = Marshal.from_channel ic in
  close_in ic;
  crf_model

let main () =
  let tst_dir = ref "" 
  and tst_regstr = ref ".*" 
  and input_file = ref ""
  and model_file = ref "" 
  and outsuf = ref "" 
  and output_file = ref ""
  and lex_dir = ref ""
  and feature_set = ref ""
  and begin_encode = ref false
  and seq_boundary = ref "s" in
  Arg.parse
    [("-input-dir", Arg.Set_string(tst_dir), "Input directory containing training files");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-input-file", Arg.Set_string(input_file), " Input file");
     ("-filter", Arg.Set_string(tst_regstr), "Input regexp filter");
     ("-outsuffix", Arg.Set_string (outsuf), "Suffix for tagged files");
     ("-output-file", Arg.Set_string(output_file), " Output file");
     ("-begin", Arg.Set(begin_encode), "Encode beginning of each phrase with begin label");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set to use");
     ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons");
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    "Usage: segment -input-dir <input dir> -model <model to use> -filter <regexp> -outsuffix <file suffix> -lexicon-dir <lex dir>";
  let crf_model = get_model !model_file in
  let lex = 
    if !lex_dir = "" then None else 
      let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
	Some (TextFeatures.Lexicon.build_lexicons_txt paths) in
  let tst_path,in_p = if !input_file = "" then !tst_dir,false else !input_file,true in
    MySgmlfront.run_on_sgml ~tst_filter:(Str.regexp (!tst_regstr)) ~add_begin:(!begin_encode) 
      ~input_file:in_p ~output_file:!output_file !seq_boundary tst_path !outsuf crf_model !feature_set lex;;

main ();;
