(**
  Copyright The MITRE Corporation 2005.  All rights reserved.
*)


module MySeq = Sequences.Make(PosMgr)
module MySgmlfront = Sgmlfront.Make(MySeq)

let get_model model_file =
  let ic = open_in_bin model_file in
  let (crf_model: MySgmlfront.tag Crf.user_crf_model) = Marshal.from_channel ic in
    close_in ic;
    crf_model
  
let main () =
  let tr_dir = ref ""
  and tr_regstr = ref ".*"
  and in_file = ref ""
  and model_file = ref ""
  and tagset = ref "" 
  and lex_dir = ref ""
  and begin_encode = ref false 
  and feature_set = ref ""
  and compact = ref false
  and max_its = ref 1000 
  and vp_train = ref false
  and hybrid_train = ref false
  and from_mask = ref ""
  and gp = ref 100.0
  and seq_boundary = ref "s"
  and intermediate = ref (-1) in
    Arg.parse
      [("-input-dir", Arg.Set_string(tr_dir), "Input directory containing training files");
       ("-input-file", Arg.Set_string(in_file), " Input file");
       ("-filter", Arg.Set_string(tr_regstr), "Input regexp filter");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-tagset", Arg.Set_string(tagset), "Tagset to train against");
     ("-compact", Arg.Set(compact), "Compact training (slower, but conserves memory)");
     ("-gaussian-prior", Arg.Float (fun x -> Crf.invSigSquare := (1. /. (x *. x))), "Gaussian prior variance");
     ("-max-iters", Arg.Set_int (max_its), "Max number of iterations (default 1000)");
     ("-vp-train", Arg.Set(vp_train), " Use voted percepton training");
     ("-hybrid-train", Arg.Set(hybrid_train), " Use hybrid vp/crf training");
     ("-from-mask", Arg.Set_string(from_mask), " Train using initial feature mask");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set to use");
     ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons");
     ("-intermediate", Arg.Set_int(intermediate), " Dump out intermediate modesl (every 50 iterations)");
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    ("Current allowed feature sets: "^(String.concat " " PosMgr.feature_sets));
  let ts = MySgmlfront.gather_tagset !tagset in
  let lex = 
    if !lex_dir = "" then None else 
      let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
	Some (TextFeatures.Lexicon.build_lexicons_txt paths) in
  let inter_f = if !intermediate >= 0 then Some !model_file else None in
  let tr_path,in_p = if !in_file = "" then !tr_dir,false else !in_file,true in
    Printf.printf "\n Training from sgml";
    let ttype = 
      if !hybrid_train then MySgmlfront.Hybrid else if !vp_train then MySgmlfront.Vp 
      else if not (!from_mask = "") then MySgmlfront.CrfMask (get_model !from_mask)
      else MySgmlfront.Crf in
    let user_crf = 
      MySgmlfront.train_from_sgml ~i_file_root:inter_f ~train_type:ttype ~compact:(!compact) ~max_its:(!max_its) 
       ~tr_filter:(Str.regexp (!tr_regstr)) ~input_file:in_p 
	
       ~add_begin:(!begin_encode) !seq_boundary tr_path !feature_set ts lex in
  Crf.marshal_crf !model_file user_crf;;

main ();;
