# -*- perl -*-

sub Precision {
    my ( $h, $m) = @_;

    return 100 if (!defined($h) || $h == 0);
    return 0 if (!defined($m) || $m == 0);
    return (100*$m/$h);
}

sub Recall {
    my ( $r, $m) = @_;

    return 100 if (!defined($r) || $r ==0);
    return 0 if (!defined($m) || $m == 0);
    return (100*$m/$r);
}

sub Fm {
    my ($r, $h, $m) = @_;

    my ( $P, $R);

    $P=Precision($h,$m);
    $R=Recall($r,$m);

    return 0 if ($P+$R == 0);

    return (2.0 * ($P * $R)/
	          ($P + $R)   );
}

die <<ENDARGS

    Usage :

    $0 reference.sgml hypothesis.sgml s,\sEID=.*,,go TAG1 TAG2 TAG3 ...

Will report precision, recall, F-measure on all tags, as well as
    combined P, R, and F for the tags specified on the command line.

Character offsets are used to compare correctness.

ENDARGS
    unless (@ARGV >= 3);

undef $/;  # slurp mode - whole file at once

$rfile = shift @ARGV;
$hfile = shift @ARGV;
$pruneexp = shift @ARGV;

while ($_=shift @ARGV) {
    # combined score tags
    $Ctags{$_}=1;
}

# slurp in reference tag spans
open(F,$rfile) || die "can't read from $rfile.\n";
print STDERR "Loading reference...";
$rstring = <F>;
close(F);
@rarray = split(/(<[^>]+>)/,$rstring);
@position = ();
$offset = 0;
$widesttag = 0;
foreach (@rarray) {
    # if close tag, then pop position of open tag
    if (m,^</,) {
	$start = pop @position;
	$tag = pop @position;
	if ($pruneexp) {
	    eval '$tag =~ '.$pruneexp.';';
	}
	# normalize the tag
	$tag=tagnormalize($tag);
	$widesttag = max($widesttag,length($tag));
	$Rset{"$tag%%%$start%%%$offset"}=1;
    }
    # if open tag, push position
    elsif (m,^<,) {
	chop($_);
	$_=substr($_,1);
	push @position,$_;
	push @position,$offset;
    }
    # if not tag, increment position by length
    else {
	$offset += length($_);
    }
    print STDERR "." if ($dotcounter++ % 10000 == 0);
}
print STDERR "done.\n";



# slurp in hypothesis tag spans
print STDERR "Loading hypothesis...";
open(F,$hfile) || die "can't read from $hfile.\n";
$rstring = <F>;
close(F);
@rarray = split(/(<[^>]+>)/,$rstring);
@position = ();
$offset = 0;
foreach (@rarray) {
    # if close tag, then pop position of open tag
    if (m,^</,) {
	$start = pop @position;
	$tag = pop @position;
	if ($pruneexp) {
	    eval '$tag =~ '.$pruneexp.';';
	}
	$tag=tagnormalize($tag);
	$widesttag = max($widesttag,length($tag));
	$Hset{"$tag%%%$start%%%$offset"}=1;
    }
    # if open tag, push position
    elsif (m,^<,) {
	chop($_);
	$_=substr($_,1);
	push @position,$_;
	push @position,$offset;
    }
    # if not tag, increment position by length
    else {
	$offset += length($_);
    }
    print STDERR "." if ($dotcounter++ % 10000 == 0);
}
print STDERR "done.\n";


# compare

foreach (keys %Rset) {
    ($tag,$start,$end)=split("%%%",$_);
    $match = 0;
    $match = 1 if ($Hset{$_});
    $tagRcount{$tag}++;
    $tagMatch{$tag}+= $match;
    $tags{$tag}=1;
}

foreach (keys %Hset) {
    ($tag,$start,$end)=split("%%%",$_);
    $tagHcount{$tag}++;
    $tags{$tag}=1;
}

# print results
print "\n";
printf ("%${widesttag}s %12s%12s%12s%7s%7s%7s\n",
	"tag","Rcount","Hcount","Match",
	"Prec","Rec","Fmeas");
printf ("%${widesttag}s %12s%12s%12s%7s%7s%7s\n",
	"-----","-----","-----","-----",
	"-----","-----","-----");
foreach (keys %tags) {
    $tagid = $_;
    $tagid =~ s,\s.*,,go;
    printf ("%${widesttag}s%1s%12d%12d%12d%7.2f%7.2f%7.2f\n",
	    $_,
	    ($Ctags{$tagid})?"*":" ",
	    $tagRcount{$_},
	    $tagHcount{$_},
	    $tagMatch{$_},
	    Precision($tagHcount{$_},$tagMatch{$_}),
	    Recall($tagRcount{$_},$tagMatch{$_}),
	    Fm($tagRcount{$_},$tagHcount{$_},$tagMatch{$_})
	    );
    if ($Ctags{$tagid}) {
	$tag=$_;
	$tagRCcount+=$tagRcount{$tag};
	$tagHCcount+=$tagHcount{$tag};
	$tagMCcount+=$tagMatch{$tag};
    }
}
printf ("%${widesttag}s %12s%12s%12s%7s%7s%7s\n",
	"-----","-----","-----","-----",
	"-----","-----","-----");
printf ("%${widesttag}s%1s%12d%12d%12d%7.2f%7.2f%7.2f\n",
	"* selected"," ",
	$tagRCcount,$tagHCcount,$tagMCcount,
	Precision($tagHCcount,$tagMCcount),
	Recall($tagRCcount,$tagMCcount),
	Fm($tagRCcount,$tagHCcount,$tagMCcount));
print "\n";

sub max {
    my $r = shift;
    while (@_) {
	$r = ($_[0] > $r) ? $_[0] : $r;
	shift;
    }
    return $r;
}

sub tagnormalize {
    my $t = shift;
    my @x;
    my @y;
    my $ttype;

    $t = uc($t);
    @x = split(/\s+/,$t);
    $ttype = shift @x;
    @y = sort {$a cmp $b} @x;
    return join(' ',$ttype,@y);
}
