let get_model model_file = 
  let in_ch = open_in_bin model_file in
  let (model:string option Crf.user_crf_model) = Marshal.from_channel in_ch in
    close_in in_ch;
    model;;


let get_singles file =
  let lines = Crfdistance.get_lines (Str.regexp "\t") file in
  List.map (fun li -> match li with a :: _ -> a | _ -> failwith "Invalid line") lines


let main () =
  let single_file = ref ""
  and model_file = ref ""
  and nbest = ref 1 
  and feature_set = ref "EDIT-DIST"
  and generated_file = ref "" in
    Arg.parse
      [("-single-file", Arg.Set_string(single_file), " Single file");
       ("-model-file", Arg.Set_string(model_file), " Model file");
       ("-feature-set", Arg.Set_string(feature_set), " Feature set");
       ("-nbest", Arg.Set_int(nbest), " Number of generated forms to produce");
       ("-generated-forms-file", Arg.Set_string(generated_file), " File containing generated forms")]
      (fun x -> ())
      "Usage: crfdistance_score -pair-file <pair file> -model-file <model file> -scores <score file>";
    let crf = get_model !model_file in
    let singles = get_singles !single_file in
    let forms = Crfdistance.top_n_forms crf !feature_set !nbest singles in
    let ostr = open_out !generated_file in
      List.iter (fun (s,forms) -> 
		   Printf.fprintf ostr "%s -> " s;
		   Array.iter (fun x -> Printf.fprintf ostr "\n\t%s" x) forms;
		   Printf.fprintf ostr "\n\n") forms;;

main ();;
