let get_model model_file = 
  let in_ch = open_in_bin model_file in
  let (model:string option Crf.user_crf_model) = Marshal.from_channel in_ch in
  close_in in_ch;
  model;;

let get_pairs file =
  let lines = Crfdistance.get_lines (Str.regexp "\t") file in
  List.rev_map (fun li -> match li with a :: b :: _ -> a,b | _ -> failwith "Invalid line") lines

let main () =
  let pair_file = ref ""
  and model_file = ref ""
  and best_list = ref false 
  and feature_set = ref "EDIT-DIST"
  and scores_file = ref "" in
    Arg.parse
      [("-pair-file", Arg.Set_string(pair_file), " Pair file");
       ("-model-file", Arg.Set_string(model_file), " Model file");
       ("-best-list", Arg.Set(best_list), " Select and score best from a list");
       ("-feature-set", Arg.Set_string(feature_set), " CRF feature set to use");
       ("-scores-file", Arg.Set_string(scores_file), " File containing scores for the pair")]
      (fun x -> ())
      "Usage: crfdistance_score -pair-file <pair file> -model-file <model file> -scores <score file>";
    let crf = get_model !model_file in
      Crf.invSigSquare := 0.0; (* don't penalize log-likelihood *)
      if not (!best_list) then (
	let pairs = get_pairs !pair_file in
	let scores = List.rev (Crfdistance.score_pairs (Crfdistance.init crf !feature_set) pairs) in
	let ostr = open_out !scores_file in
	  List.iter2 (fun (s1,s2) sc -> Printf.fprintf ostr "%f\t%s\t%s\n" sc s1 s2) pairs scores;
	  close_out ostr)
      else 
	let lines = Crfdistance.get_lines (Str.regexp "\t") !pair_file in
	let pair_crf = (Crfdistance.init crf !feature_set) in
	let ostr = open_out !scores_file in
	let ll = ref 0. in
	let best_score = ref neg_infinity in
	let best_match = ref "" in
	let avg_score = ref 0. in
	let ll_avg = ref 0. in
	  List.iter
	    (fun x ->
	       match x with
		   a :: b ->
		     best_score := neg_infinity;
		     avg_score := 0.;
		     List.iter
		       (fun y ->
			  let s = -. (Crfdistance.eval_pair pair_crf a y) in
			    avg_score := !avg_score +. s;
			    if s > !best_score then (best_score := s; best_match := y)) b;
		     ll := !ll +. !best_score;
		     ll_avg := !ll_avg +. !avg_score;
		     Printf.fprintf ostr "%f\t%s\t%s\n" !best_score a !best_match
		 | _ -> ()) lines;
	  Printf.printf "\n\n Test Log Likelihood: %f" !ll;
	  Printf.printf "\n\n Expected Test Log Likelihood: %f\n\n" !ll_avg;
	  close_out ostr;;

main ();;
