let marshal_model (file: string) (model: Crfdistance.transition_label Crf.user_crf_model) =
  let o_ch = open_out_bin file in
    Marshal.to_channel o_ch model [];
    close_out o_ch;;

let main () =
  let pair_file = ref ""
  and model_file = ref "" 
  and max_its = ref 1000
  and compact = ref false 
  and vp_train = ref false 
  and intermediate = ref false
  and feature_set = ref "EDIT-DIST" in
    Arg.parse
      [("-pair-file", Arg.Set_string(pair_file), " Input file with pairs of strings, tab separated");
       ("-model-file", Arg.Set_string(model_file), " Resulting trained CRF for string similarity");
       ("-max-its", Arg.Set_int(max_its), " Maximum number of iterations (default 1000)");
       ("-gaussian-prior", Arg.Float(fun x -> Crf.invSigSquare := (1. /. (x *. x))), "Gaussian prior variance");
       ("-vp-train", Arg.Set(vp_train), " Use Voted Perceptron to train crf");
       ("-intermediate", Arg.Set(intermediate), " Produce intermediate results to models");
       ("-feature-set", Arg.Set_string(feature_set), " Set of features to use (default EDIT-DIST) ");
       ("-compact", Arg.Set(compact), " Compact training option")]
      (fun x -> ())
      "Usage: crfdistance_train -pair-file <file of string pairs> -model-file <resultsing model file>";
    let inter_f = if !intermediate then Some !model_file else None in
    let crf,_ = Crfdistance.train_on_seqs ~i_file_root:inter_f ~compact:(!compact) ~max_its:(!max_its) ~vp_train:(!vp_train) 
		  !feature_set !pair_file in
      marshal_model !model_file crf;;

main ();;


