let feature_sets = ["EDIT-DIST"; "EDIT-DIST-2"]

type t = string

let of_string s = s

exception IsFound

let within_last_word dseq p =
  let len = Array.length dseq in
    try
      for i = p to len - 1 do
	match FeatureGen.Source.get_obs dseq.(i) with 
	    Some " " -> raise IsFound
	  | _ -> () done;
      FeatureGen.set_fs ([("LastWord",1.0)],false)
    with IsFound -> raise FeatureGen.Feature_failure

let within_parens dseq p =
  let len = Array.length dseq in
    try
      for i = p to len - 1 do
	match FeatureGen.Source.get_obs dseq.(i) with 
	    Some ")" -> raise IsFound
	  | _ -> () done;
      FeatureGen.set_fs ([("WithinParens",1.0)],false)
    with IsFound -> raise FeatureGen.Feature_failure

let string_distance d =
  [TextFeatures.wd_f;
   TextFeatures.edge_f;
   TextFeatures.node_f;
   TextFeatures.regexp_f_gen "SINGLE_DIGIT" (Str.regexp "[0-9]$");
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [-1;0];
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [0;1]]

let string_distance_2 d =
  [TextFeatures.wd_f;
   TextFeatures.edge_f;
   TextFeatures.node_f;
   TextFeatures.regexp_f_gen "SINGLE_DIGIT" (Str.regexp "[0-9]$");
   TextFeatures.regexp_f_gen "PUNCT" (Str.regexp "^[^A-Za-z0-9]+$");
   within_last_word;
   within_parens;
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [-1;0];
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [0;1]]


let string_distance_3 d =
  [TextFeatures.wd_f;
   TextFeatures.edge_f;
   TextFeatures.node_f;
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [-1;0];
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [-2;-1;0];
   TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-2;-1;0];
   TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0];
   TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [-1;0;1];
   TextFeatures.n_gram_fn ~edge:true TextFeatures.wd_f [0;1];
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [-1;0;1];
   TextFeatures.n_gram_fn ~edge:false TextFeatures.wd_f [0;1];
   TextFeatures.regexp_f_gen "PUNCT" (Str.regexp "^[^A-Za-z0-9]+$");
   TextFeatures.regexp_f_gen "SINGLE_DIGIT" (Str.regexp "[0-9]$")]

let get_feature_fns feature_set data =
  match feature_set with
    | "EDIT-DIST" -> string_distance data
    | "EDIT-DIST-2" -> string_distance_2 data
    | "EDIT-DIST-3" -> string_distance_3 data
    | _ -> failwith "Unknown feature set...\n\n"
