(*
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(** 
    A simple gradient-based approach to Feature Induction.  The idea
    is to select features that have the greatest local gradient components - 
    i.e., the emprical feature expectations and model feature expectations
    have a large difference.  

    @author Ben Wellner

    @see "Feature Induction." Efficiently Inducing Features of Conditional
      Random Fields.  Andrew McCallum, Conference on Uncertainty in Artificial Intelligence
       (UAI) 2003.

*)

val set_beam_size : int -> unit
val set_num_rounds : int -> unit

(** This function is used at Runtime to apply an induced complex feature mask
    over the atomic extracted features.  *)
val augment_tst_data :
  FeatureGen.crf_instance array array ->
  'a ->
  (FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t ->
  (FeatureGen.feature_name, FeatureGen.feature_name) Hashtbl.t ->
  FeatureGen.crf_instance array array

val get_new_tr_data :
  FeatureGen.crf_instance array array ->
  'a FeatureGen.Alphabet.t ->
  FeatureGen.AgglFeatureSet.t ->
  (FeatureGen.feature_name, FeatureGen.feature_name) Hashtbl.t ->
  FeatureGen.crf_instance array array *
  (int * int * FeatureGen.feature_name) FeatureGen.Alphabet.t *
  (FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t

(** Same as get_new_tr_data but uses a hashtbl instead of set *)
val augment_tr_data :
  FeatureGen.crf_instance array array ->
  'a FeatureGen.Alphabet.t ->
  (FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t ->
  (FeatureGen.feature_name, FeatureGen.feature_name) Hashtbl.t ->
  FeatureGen.crf_instance array array *
  (int * int * FeatureGen.feature_name) FeatureGen.Alphabet.t *
  (FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t

(** Top-level training function that incoprorates gradient-based feature induction. *)
val induce_crf_features :
  ?intermediate:int ->
  ?la_fset_d:(string * (FeatureGen.AgglFeatureSet.elt, 'a) Hashtbl.t *
	      'b FeatureGen.Alphabet.t) option -> 
  ?compact:bool ->
  ?max_its:int ->
  (FeatureGen.AgglFeatureSet.elt, 'a) Hashtbl.t ->
  'b FeatureGen.Alphabet.t ->
  FeatureGen.crf_instance array array ->
  Crf.model *
  ((FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t *
   (FeatureGen.feature_name, FeatureGen.AgglFeatureSet.elt) Hashtbl.t)
  option
