(**
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)



#use "topfind";;
#require "crf dumper";;

open Crf
open Sgmlfront
open Dumper (* scary magic that will print out any type as a string *)

let get_model model_file =
  let ic = open_in model_file in
  let crf_model = Marshal.from_channel ic in
  close_in ic;
  crf_model


let print_crf_model_details user_crf str =
  match user_crf with
      {crf=crf; crf_fset=fset; crf_la=la; fset_mask=mask} ->
	let inv_la = FeatureGen.Alphabet.get_inverse la in
	let main_fset = match mask with None -> fset | Some(fset,_) -> fset in
	  Printf.printf "Number of parameters: %d\n" (Bigarray.Array1.dim crf.Crf.parameters);
	  Printf.fprintf str "\n Labels: "; flush stdout;
	  Hashtbl.iter
	    (fun i l ->
	       Printf.fprintf str "(id=%d -> %s)\n" i (dump l)) 
	    (FeatureGen.InvAlphabet.get_inv_map inv_la); flush str;
	  Hashtbl.iter 
	    (fun k v ->
	       let fset_details = v.FeatureGen.FeatureType.fdetail in
	       let fname = FeatureGen.flatten_feature_name k in
		 Printf.fprintf str "%s: \n" fname; flush str;
		 FeatureGen.FeatureSet.iter
		   (fun (yprev,yp,f,v) ->
		      Printf.fprintf str "\t(%s,%s,%f) = %f [%d]\n"
		      (if yprev > -1 then
			 let x = (FeatureGen.InvAlphabet.get inv_la yprev) in
			   dump x
		      else "NULL")
		      (dump (FeatureGen.InvAlphabet.get inv_la yp))
		      v crf.parameters.{f} f) fset_details) main_fset

let _ =
  let ostr = open_out Sys.argv.(2) in
    print_crf_model_details (get_model Sys.argv.(1)) ostr;
    close_out ostr
