(**
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(** This is main routine for launching text segmentation/labeling tasks. *)

module MySeq = Sequences.Make(Feature_manager)
module MySgmlfront = Sgmlfront.Make(MySeq)

let get_model model_file =
  let ic = open_in model_file in
  let (crf_model: MySgmlfront.tag Crf.user_crf_model) = Marshal.from_channel ic in
  close_in ic;
  crf_model

let main () =
  let tst_dir = ref "" 
  and tst_regstr = ref ".*" 
  and model_file = ref "" 
  and outsuf = ref "" 
  and lex_dir = ref ""
  and feature_set = ref ""
  and begin_encode = ref false
  and seq_boundary = ref "s" in
  Arg.parse
    [("-input-dir", Arg.Set_string(tst_dir), "Input directory containing training files");
     ("-model", Arg.Set_string(model_file), "Output model file");
     ("-filter", Arg.Set_string(tst_regstr), "Input regexp filter");
     ("-outsuffix", Arg.Set_string (outsuf), "Suffix for tagged files");
     ("-begin", Arg.Set(begin_encode), "Encode beginning of each phrase with begin label");
     ("-feature-set", Arg.Set_string(feature_set), "Feature set associated with task (POS, XG, XG-NOPOS, etc.)");
     ("-lexicon-dir", Arg.Set_string(lex_dir), "Directory containing lexicons");
     ("-seq-boundary", Arg.Set_string(seq_boundary), "Sequence boundary annotation")]
    (fun x -> ())
    "Usage: segment -input-dir <input dir> -model <model to use> -filter <regexp> -outsuffix <file suffix> -lexicon-dir <lex dir>";
  let crf_model = get_model !model_file in
  let paths = Array.map (fun x -> (!lex_dir)^x) (Sys.readdir !lex_dir) in
  let lex = TextFeatures.Lexicon.build_lexicons_txt paths in
    MySgmlfront.run_on_sgml ~tst_filter:(Str.regexp (!tst_regstr)) ~add_begin:(!begin_encode) !seq_boundary 
      !tst_dir !outsuf crf_model !feature_set (Some lex);;

main ();;
