(*
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(**

  Voted Perceptron algrorithm, an approximate method for training a CRF.
  This implements Voted Perceptron following Collins 2002.  The final parameters
  are the {e averaged} parameter values over all epochs.  Implementation is
  not yet optimized.

  @author Ben Wellner

  @see "Michael Collins" Michael Collins.  Discriminative Training Methods for Hidden Markov Models: Theory and Experiments with Perceptron Algorithms.  EMNLP 2002

*)


(** Traing using Voted Perceptron.  [int] specifies the number of features while
   [int] specifies the number of labels.  [FeatureGen.train_data] specifies the
   training data *)
val vp_train :
  ?la_fset_d:(string *
              (FeatureGen.feature_name, FeatureGen.FeatureType.t) Hashtbl.t *
              'a FeatureGen.Alphabet.t)
             option ->
  ?compact:bool ->
  ?max_its:int -> int -> int -> FeatureGen.train_data -> Crf.model
(** @return The function returns a [Crf.model] *)

(** Initial versino of {e hybrid training} in which Voted Perceptron is used 
    first to find a set of very good initial parameters and then log-likelihood
    training takes over.  Works, but better scaling of parameters is required
    for much gain to be realized. *)
val hybrid_train :
    ?compact:bool ->
    ?vp_its:int ->
    ?crf_its:int ->
  ?la_fset_d:(string *
              (FeatureGen.feature_name, FeatureGen.FeatureType.t) Hashtbl.t *
              'a FeatureGen.Alphabet.t)
             option ->
    'a FeatureGen.Alphabet.t ->
    (FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t ->
    int -> int -> FeatureGen.train_data -> 
  (Crf.model * 
   ((FeatureGen.feature_name, FeatureGen.FeatureTypeSet.elt) Hashtbl.t *
    (FeatureGen.feature_name, FeatureGen.AgglFeatureSet.elt) Hashtbl.t)
     option)
