(* a new reader that allows for arbitrary strings as features and deals with all the symbol
   mapping *)


type protoseq = {lab: string;
		 features: (string * float) array;
		 comment: string}

let r1 = Str.regexp " +"
let r2 = Str.regexp ":"
let r3 = Str.regexp "[^0-9\\.]" (*everything but digits and periods *)

let rec get_features flist acc =
  match flist with
      a :: b ->
	if a.[0] = '#' then
	  List.rev acc,(String.concat " " b)
	else ( 
	  try
	    (match (Str.split r2 a) with
	       | [fname;fval] ->
		   let afval = Str.global_replace r3 "" fval in
		     get_features b ((fname,(float_of_string afval)) :: acc)
	       | [fname] -> get_features b ((fname,1.0) :: acc)
	       | _ -> failwith "Invalid feature vector")
	  with Failure "float_of_string" -> Printf.printf "Failed on %s\n" a; failwith "bad conversion" )
    | [] -> List.rev acc,"" 

let protoseqs_from_string_list stringlist = 
  List.map
    (fun str ->
       match Str.split r1 str with
	   label :: features ->
	     let nfeas,comment = get_features features [] in
	       {lab=label; features=Array.of_list nfeas;comment=comment}
	 | _ -> failwith "Invalid feature vector") stringlist

let protoseqs_from_vectors vecfile = 
  let istr = open_in vecfile in
  let insts = ref [] in
  let rec loop () =
    match Str.split r1 (input_line istr) with
	"++" :: _  | [] -> loop ()
      | label :: features ->
	  let nfeas,comment = get_features features [] in
	    insts := 
	    {lab=label;
	     features= Array.of_list nfeas; comment=comment} :: !insts;
	    loop () in
    try 
      loop ()
    with End_of_file ->
      List.rev !insts

let get_inst_set ?(runtime=false) ?(fa=None) ?(la=None) protos =
  let insts = 
    List.map 
      (fun p -> 
	 let inst = Instance.create_user_inst p.lab None in
	   Array.iter (fun (fn,v) -> Instance.add_feature inst fn v) p.features;
	   inst) protos in
  let iset = 
    if runtime then
      let la = match la with Some la -> la | None -> failwith "label table needed" in
      let fa = match fa with Some fa -> fa | None -> failwith "feature table needed" in
	Instance.create_instance_set ~fa:fa ~la:la true
    else
      Instance.create_instance_set false in
    Instance.intern_insts_inc insts iset;
    let fa = iset.Instance.falpha and la = iset.Instance.lalpha in
      (iset,fa,la)

