(*
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)

module FType : sig type t = string * float val compare : 'a * 'b -> 'a * 'c -> int end

module FSet : 
  sig
    type elt = FType.t
    type t = Set.Make(FType).t
    val iter : (elt -> unit) -> t -> unit
  end

type 'a raw_instance = {mutable label: int;
			f_vector: Vectors.sparse_vector;
			source: 'a} (* source represents information to save of same type as info *)

type 'a user_instance = {lab: string;
			 mutable features: FSet.t;
			 info: 'a}

type alphabet = {map: (string,int) Hashtbl.t;
		 mutable size: int;
		 counts: (int,int) Hashtbl.t}

type inv_alphabet = {inv_map: (int,string) Hashtbl.t;
		     mutable inv_size: int}

type 'a instance_set = {mutable r_insts: 'a raw_instance list;
			falpha: alphabet;
			lalpha: alphabet;
			mutable fixed: bool}

val create_alphabet : unit -> alphabet
val copy_inst_list : 'a user_instance list -> 'a user_instance list
val print_user_inst : ?add_str:string -> 'a user_instance -> out_channel -> unit
val split_set : ?rand:bool -> ?f1:bool -> ?f2:bool -> 'a instance_set -> float -> ('a instance_set * 'a instance_set)
val create_raw : int -> Vectors.sparse_vector -> 'a -> 'a raw_instance
val create_user_inst : string -> 'a -> 'a user_instance
val create_instance_set : ?fa:alphabet -> ?la:alphabet -> bool -> 'a instance_set

val ts_add : 'a instance_set -> 'a raw_instance -> unit
val add_feature : 'a user_instance -> string -> float -> unit
val add_feature_l : 'a user_instance -> string list -> float -> unit

val get_f_vector : 'a raw_instance -> Vectors.sparse_vector
val get_label : 'a raw_instance -> int

val intern_insts_inc : ?add_dummies:bool -> ?src_fn:('a -> 'a) -> 'a user_instance list -> 'a instance_set -> unit
val invert_alphabet : alphabet -> inv_alphabet

val intern_inst_seq : ?src_fn:('a -> 'a) -> 'a user_instance list -> 'a instance_set -> unit

val single_unlabeled_inst : 'a user_instance -> alphabet -> 'a raw_instance

val commonify_instances : 'a instance_set -> int -> 'a instance_set
