(*
  Copyright The MITRE Corporation 2005.  All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*)


open Str

val gaussian_prior : float ref
type 'a il = 'a Instance.raw_instance array

type 'a grad_args = { ilist: 'a il;
		      num_features: int;
		      constraints: Vectors.rbarray;
		      num_labels: int}

and  model = {parameters: Vectors.rbarray;
	      nfs: int;
	      nls: int}

(** This record represents the full user-level model - i.e. it includes the 
    mappings from feature names to ids *)
type user_model = { um_model: model;
		    um_falpha:  Instance.alphabet;
		    um_lalpha:  Instance.alphabet }

val evaluate_instances_x : model -> 'a il -> float
val evaluate_instances : model -> 'a Instance.instance_set -> float
val classify_instances_raw : model -> 'a il -> (int * float) array
val classify_instances : model -> 'a il -> int array
val get_distributions : model -> 'a il -> float array array
val precision_recall_x : int -> model -> 'a il -> (float * float)
val train_lbfgs_x : ?no_print:bool -> ?max_iters:int -> 'a il -> int -> int -> model
val train_lbfgs : ?no_print:bool -> 'a Instance.instance_set -> model
val score_arrays : int array -> int array -> float
val pretty_print_precision_recall : ?tabs:bool -> model -> 'a il -> 
  (int,string) Hashtbl.t -> unit
val get_precision_recall : model -> 'a il -> (int,string) Hashtbl.t ->
  (string * float * float * float * int) list
val get_best_class : model -> 'a Instance.raw_instance -> (int * float)

val conf_matrix : int array -> int array -> int -> float array array
val print_confusion_matrix : float array array -> (int,string) Hashtbl.t -> unit
val class_scores_unnormalized : int -> int -> Vectors.rbarray -> Vectors.sparse_vector -> float array
val compute_constraints : 'a il -> int -> int -> Vectors.rbarray
